'''
onshapepy
=========

Demonstrates usage of API keys for the Onshape REST API
'''

__copyright__ = 'Copyright (c) 2016 Onshape, Inc.'
__license__ = 'All rights reserved.'
__title__ = 'onshapepy'
__all__ = ['onshape', 'client', 'utils']
